#!/usr/bin/perl

use Getopt::Std;
use Mysql;
use Time::Local;


# Opts:
#	h = DB-Host
#	b = Database
#	u = DB-User
#	p = passwd
#	a = HostAddress
#	s = Service Status
#	A = HostAilias
#	d = ServiceDescription
#	o = output
#	t = Datetime
#       T = STATETYPE


#
# Get the options
#
getopts("h:b:u:p:a:s:A:d:o:t:T:");

$db_host = $opt_h;
$db      = $opt_b;
$db_user = $opt_u;
$db_pass = "";

#
# Convert the time given by $opt_t to UNIX_TIMESTAMP and DATETIME (sql) format
# by calling the get_time() subroutine
#
%now = &get_time($opt_t);

#
# Building the SQL-statment for the states CRITICAL and WARNING
#
if (($opt_s eq "CRITICAL") || ($opt_s eq "WARNING") || ($opt_s eq "DOWN") || ($opt_s eq "UNREACHABLE")) {
    $sql_statement = "INSERT INTO TBFailureStatistics
			VALUES
			    ('$now{'date'}','$opt_A','$opt_a','$opt_d','$opt_s','$opt_o','','')\;";
}

#
# Building the SQL-statment for the state RECOVERY
#
if (($opt_s eq "RECOVERY") || ($opt_s eq "UP")) {

    $recoveryDate = $now{'date'};
    %down         = &get_down_date; # Get the time and date when the service went down
    $up           = $now{'time'};    
    
    #
    # Calculating the downtime
    #
    $downtime = ($up - $down{'time'});

    $down_date = $down{'date'}; 

    $sql_statement = "UPDATE TBFailureStatistics
			SET
			    RecoveryDate='$recoveryDate',
			    DowntimeSec='$downtime'
			WHERE
			    FailureDate='$down_date'
			    AND HostDescription='$opt_A'
			    AND IPAddr='$opt_a'
			    AND Service='$opt_d'
			    AND RecoveryDate=''\;";
}

#
# Make an entry in the Database
#
print("st: $sql_statement\n");
if ((($opt_s eq "CRITICAL") || ($opt_s eq "RECOVERY") || ($opt_s eq "DOWN") || ($opt_s eq "UNREACHABLE") || ($opt_s eq "UP") || ($opt_s eq "WARNING")) &&
    ($opt_T eq "HARD")) {
    $dbh = Mysql->connect($db_host, $db, $db_user, $db_pass);
    $sth = $dbh->query($sql_statement);
}
######################################################################################
#
#				 Start Subroutines 
#
######################################################################################

sub get_time() {

    my $date_str;
    my @date_arr;
    my $mon;
    my $mday;
    my $year;
    my @time_arr;
    my $hours;
    my $min;
    my $sec;
    my $time;
    my $date;
    my $retval;
    
    $date_str = $_[0];
    @date_arr = split(/ /, $date_str);

    $mon   = $date_arr[1];
    $mday  = $date_arr[2];
    $year  = $date_arr[4];

    @time_arr  = split(/:/, $date_arr[3]);
    $hours     = $time_arr[0];
    $min       = $time_arr[1];
    $sec       = $time_arr[2];

    if ($mday < 10) { $day = "0".$mday; }

    $mon =~ s/Jan/01/;
    $mon =~ s/Feb/02/;
    $mon =~ s/Mar/03/;
    $mon =~ s/Apr/04/;
    $mon =~ s/May/05/;
    $mon =~ s/Jun/06/;
    $mon =~ s/Jul/07/;
    $mon =~ s/Aug/08/;
    $mon =~ s/Sep/09/;
    $mon =~ s/Oct/10/;
    $mon =~ s/Nov/11/;
    $mon =~ s/Dec/12/;

    $time = timelocal($sec, $min, $hours, $mday, $mon, $year);
    $date = "$year-$mon-$mday $date_arr[3]";
    %retval = ("time",$time,"date",$date);

    return(%retval);
}

sub get_down_date() {

    my $dbh;
    my $sth;
    my $date;
    my $time;
    my @buff;
    my @date_arr;
    my @time_arr;
    my $retval;

    $dbh = Mysql->connect($db_host, $db, $db_user, $db_pass);
    $sth = $dbh->query("SELECT max(FailureDate)
			    FROM
				TBFailureStatistics
			    WHERE
				HostDescription='$opt_A'
				AND IPAddr='$opt_a'
				AND Service='$opt_d'
				AND RecoveryDate=''\;");

    $date = $sth->fetchrow;

    @buff      = split(/ /, $date);
    @date_arr  = split(/-/, $buff[0]);
    @time_arr  = split(/:/, $buff[1]);


    $time = timelocal($time_arr[2], $time_arr[1], $time_arr[0], $date_arr[2], $date_arr[1], $date_arr[0]);

    %retval = ("time",$time,"date",$date);

    return(%retval);
}

exit 0;